%This file replicates the MCMC analyses of the paper "Optimal Decision
%Rules for Weak GMM" using data from Graddy (1995)

clear
load FishMarketData
rng(1)

% Specify quantile of interest.  Paper considers 0.75 and 0.5
tau=0.75;

alpha_bounds=[0 30];
beta_bounds=[-10 30];

%Set tuning parameters for MH chain
chain_length=10^6;

%% Code to actually estimate model

%Define outcome, endogenous regressor, and exogenous regressors
y=qty;
d=price;
x=ones(length(y),1);

%Set instruments as in Chernozhukov Hansen and Jansson
z=[mixed stormy];

obs=length(y);
moment_dim=size(x,2)+size(z,2);

%create check function
rho=@(tau_temp,u) tau_temp-(u<=0);

%create moment function
gmat=@(tau_temp,beta,alpha) repmat(rho(tau_temp,y-d*beta-x*alpha),1,size(x,2)+size(z,2)).*[x z];

%% MCMC based on CUE
%Calculate regression estimates to use as starting values
W=[d x];
%Choose starting value equal to OLS coefficient
theta_init=(W'*W)^-1*W'*y;
log_obj=@(theta) CUGMM_loglik(gmat(tau,theta(1),theta(2))) -10^10*(1-(theta(1)>=beta_bounds(1) && theta(1)<=beta_bounds(2)...
            && theta(2)>=alpha_bounds(1) && theta(2)<=alpha_bounds(2)));
[theta_store, log_obj_store]=slicesample_custom(theta_init,chain_length,'logpdf',log_obj,'width',10);
theta_store=theta_store';

%% MCMC based on EL
%Calculate regression estimates to use as starting values
W=[d x];
%Choose starting value equal to OLS coefficient
theta_init=(W'*W)^-1*W'*y;
log_obj=@(theta) EL_loglik(gmat(tau,theta(1),theta(2))) -10^10*(1-(theta(1)>=beta_bounds(1) && theta(1)<=beta_bounds(2)...
            && theta(2)>=alpha_bounds(1) && theta(2)<=alpha_bounds(2)));
[theta_store3, log_obj_store3]=slicesample_custom(theta_init,chain_length,'logpdf',log_obj,'width',10);
theta_store3=theta_store3';


%% MCMC based on ETEL
%Calculate regression estimates to use as starting values
W=[d x];
%Choose starting value equal to OLS coefficient
theta_init=(W'*W)^-1*W'*y;
log_obj=@(theta) ETEL_loglik(gmat(tau,theta(1),theta(2))) -10^10*(1-(theta(1)>=beta_bounds(1) && theta(1)<=beta_bounds(2)...
            && theta(2)>=alpha_bounds(1) && theta(2)<=alpha_bounds(2)));
[theta_store4, log_obj_store4]=slicesample(theta_init,chain_length,'logpdf',log_obj,'width',50);
theta_store4=theta_store4';


%% Generate plots
close all

figure(1)

plot_obs=500;
hist_bins=50;

subplot(3,3,1)
sample_ID=randsample(size(theta_store,2),plot_obs,false);
theta_sample=theta_store(:,sample_ID);
scatter(theta_sample(1,:),theta_sample(2,:));
title('(a) Sample from Quasi-Posterior')
ylim([0,30])
xlim([-10,30])
ylabel('\alpha','FontSize',16)
xlabel('\beta','FontSize',16)

subplot(3,3,2)
GMM=theta_store(:,log_obj_store==max(log_obj_store));
GMM=mean(GMM,2);
post_mean_CUE=mean(theta_store,2);
histogram(theta_store(2,:),hist_bins,'Normalization','pdf','Edgecolor','none')
hold on
xline(post_mean_CUE(2),'--r','Linewidth',1)
xline(GMM(2),'--k','Linewidth',2)
hold off
xlim([0,30])
xlabel('\alpha','FontSize',16)
ylabel('Density','FontSize',16)
title('(a) Quasi-Posterior for \alpha')
legend('Quasi-Posterior','Posterior Mean','GMM Estimate')

subplot(3,3,3)
GMM=theta_store(:,log_obj_store==max(log_obj_store));
GMM=mean(GMM,2);
post_mean_CUE=mean(theta_store,2);
histogram(theta_store(1,:),hist_bins,'Normalization','pdf','Edgecolor','none')
hold on
xline(post_mean_CUE(1),'--r','Linewidth',1)
xline(GMM(1),'--k','Linewidth',2)
hold off
xlim([-10,30])
xlabel('\alpha','FontSize',16)
ylabel('Density','FontSize',16)
title('(a) Quasi-Posterior for \alpha')
legend('Quasi-Posterior','Posterior Mean','GMM Estimate')

subplot(3,3,4)
sample_ID=randsample(size(theta_store,2),plot_obs,false);
theta_sample=theta_store3(:,sample_ID);
scatter(theta_sample(1,:),theta_sample(2,:));
title('(a) Sample from Quasi-Posterior')
ylim([0,30])
xlim([-10,30])
ylabel('\alpha','FontSize',16)
xlabel('\beta','FontSize',16)

subplot(3,3,5)
EL=theta_store3(:,log_obj_store3==max(log_obj_store3));
EL=mean(EL,2);
post_mean_EL=mean(theta_store3,2);
histogram(theta_store3(2,:),hist_bins,'Normalization','pdf','Edgecolor','none')
hold on
xline(post_mean_EL(2),'--r','Linewidth',1)
xline(EL(2),'--k','Linewidth',2)
hold off
xlim([0,30])
xlabel('\alpha','FontSize',16)
ylabel('Density','FontSize',16)
title('(c) Quasi-Posterior for \alpha')
legend('Quasi-Posterior','Posterior Mean','EL Estimate')

subplot(3,3,6)
EL=theta_store3(:,log_obj_store3==max(log_obj_store3));
EL=mean(EL,2);
post_mean_EL=mean(theta_store3,2);
histogram(theta_store3(1,:),hist_bins,'Normalization','pdf','Edgecolor','none')
hold on
xline(post_mean_EL(1),'--r','Linewidth',1)
xline(EL(1),'--k','Linewidth',2)
hold off
xlim([-10,30])
xlabel('\beta','FontSize',16)
ylabel('Density','FontSize',16)
title('(d) Quasi-Posterior for \beta')
legend('Quasi-Posterior','Posterior Mean','EL Estimate')

subplot(3,3,7)
sample_ID=randsample(size(theta_store,2),plot_obs,false);
theta_sample=theta_store4(:,sample_ID);
scatter(theta_sample(1,:),theta_sample(2,:));
title('(a) Sample from Quasi-Posterior')
ylim([0,30])
xlim([-10,30])
ylabel('\alpha','FontSize',16)
xlabel('\beta','FontSize',16)

subplot(3,3,8)
ETEL=theta_store4(:,log_obj_store4==max(log_obj_store4));
ETEL=mean(ETEL,2);
post_mean_ETEL=mean(theta_store4,2);
histogram(theta_store4(2,:),hist_bins,'Normalization','pdf','Edgecolor','none')
hold on
xline(post_mean_ETEL(2),'--r','Linewidth',1)
xline(ETEL(2),'--k','Linewidth',2)
hold off
xlim([0,30])
xlabel('\alpha','FontSize',16)
ylabel('Density','FontSize',16)
title('(e) Quasi-Posterior for \alpha')
legend('Quasi-Posterior','Posterior Mean','ETEL Estimate')

subplot(3,3,9)
ETEL=theta_store4(:,log_obj_store4==max(log_obj_store4));
ETEL=mean(ETEL,2);
post_mean_ETEL=mean(theta_store4,2);
histogram(theta_store4(1,:),hist_bins,'Normalization','pdf','Edgecolor','none')
hold on
xline(post_mean_ETEL(1),'--r','Linewidth',1)
xline(ETEL(1),'--k','Linewidth',2)
hold off
xlim([-10,30])
xlabel('\beta','FontSize',16)
ylabel('Density','FontSize',16)
title('(f) Quasi-Posterior for \beta')
legend('Quasi-Posterior','Posterior Mean','ETEL Estimate')


figure(2)

plot_obs=500;
hist_bins=50;

subplot(1,3,1)
sample_ID=randsample(size(theta_store,2),plot_obs,false);
theta_sample=theta_store(:,sample_ID);
scatter(theta_sample(1,:),theta_sample(2,:));
title('(a) Sample from Quasi-Posterior')
ylim([0,30])
xlim([-10,30])
ylabel('\alpha','FontSize',16)
xlabel('\beta','FontSize',16)

subplot(1,3,2)
GMM=theta_store(:,log_obj_store==max(log_obj_store));
GMM=mean(GMM,2);
post_mean_CUE=mean(theta_store,2);
histogram(theta_store(2,:),hist_bins,'Normalization','pdf','Edgecolor','none')
hold on
xline(post_mean_CUE(2),'--r','Linewidth',1)
xline(GMM(2),'--k','Linewidth',2)
hold off
xlim([0,30])
xlabel('\alpha','FontSize',16)
ylabel('Density','FontSize',16)
title('(a) Quasi-Posterior for \alpha')
legend('Quasi-Posterior','Posterior Mean','GMM Estimate')

subplot(1,3,3)
GMM=theta_store(:,log_obj_store==max(log_obj_store));
GMM=mean(GMM,2);
post_mean_CUE=mean(theta_store,2);
histogram(theta_store(1,:),hist_bins,'Normalization','pdf','Edgecolor','none')
hold on
xline(post_mean_CUE(1),'--r','Linewidth',1)
xline(GMM(1),'--k','Linewidth',2)
hold off
xlim([-10,30])
xlabel('\alpha','FontSize',16)
ylabel('Density','FontSize',16)
title('(a) Quasi-Posterior for \alpha')
legend('Quasi-Posterior','Posterior Mean','GMM Estimate')
