%This file replicates the credible sets and confidence sets reported in the paper "Optimal Decision
%Rules for Weak GMM" using data from Graddy (1995)


clear
load FishMarketData
rng(1)

alpha_bounds=[0 30];
beta_bounds=[-10 30];
grid_step=0.05;

%Specify quantile of interest
tau=0.75;

%Define outcome, endogenous regressor, and exogenous regressors
y=qty;
d=price;
x=ones(length(y),1);

%Choose instruments
z=[mixed stormy];

obs=length(y);
moment_dim=size(x,2)+size(z,2);

%Set tuning parameters for MH chain
burn_in=5*10^3;
chain_length=10^6;
tuning=0.3;

%create check function
rho=@(tau_temp,u) tau_temp-(u<=0);

%create moment function
gmat=@(tau_temp,beta,alpha) repmat(rho(tau_temp,y-d*beta-x*alpha),1,size(x,2)+size(z,2)).*[x z];
g=@(tau_temp,beta,alpha) mean(gmat(tau_temp,beta,alpha));

%create variance estimator
Sigma=@(tau_temp,beta,alpha) cov(gmat(tau_temp,beta,alpha));

%Define quasi-likelihood multiplied by prior.  Note that dropping constant
%terms since cancel out
logquasilikelihood=@(theta) -0.5*obs*g(tau,theta(1),theta(2:end,1))...
    *pinv(Sigma(tau,theta(1),theta(2:end,1)))*g(tau,theta(1),theta(2:end,1))';

%Define S statistic
S=@(theta) obs*g(tau,theta(1),theta(2:end,1))...
    *pinv(Sigma(tau,theta(1),theta(2:end,1)))*g(tau,theta(1),theta(2:end,1))';

%Calculate regression estimates to use as starting values
W=[d x];
%Choose starting value equal to OLS coefficient
theta_init=(W'*W)^-1*W'*y;
curr_log_obj=logquasilikelihood(theta_init);
curr_theta=theta_init;
theta_store=zeros(length(theta_init),chain_length);
accept_store=NaN(chain_length,1);
S_store=NaN(chain_length,1);
log_obj_store=NaN(chain_length,1);
for n=1:chain_length
    trial_theta=curr_theta+randn(length(theta_init),1)*tuning;
    trial_log_obj=logquasilikelihood(trial_theta);
    if exp(trial_log_obj-curr_log_obj)>rand ...
            && trial_theta(1)>=beta_bounds(1) && trial_theta(1)<=beta_bounds(2)...
            && trial_theta(2)>=alpha_bounds(1) && trial_theta(2)<=alpha_bounds(2)
        curr_log_obj=trial_log_obj;
        curr_theta=trial_theta;
        accept_store(n)=1;
    else
        accept_store(n)=0;
    end
    theta_store(:,n)=curr_theta;
    S_store(n)=S(curr_theta);
    log_obj_store(n)=curr_log_obj;
end


theta_store(:,1:burn_in)=[];
S_store(1:burn_in)=[];
log_obj_store(1:burn_in)=[];

post_mean=mean(theta_store,2);
post_var=var(theta_store');

%Extract CUGMM estimate as best objective value visited
Max_location=S_store==min(S_store)';
GMM=theta_store(:,Max_location)';
GMM=GMM(1,:);
%First element corresponds to slope beta, second to intercept alpha

%% Calculate robust confidence sets
WAP_sample_size=10^4;
CV_sims=10^4;

xi_draws=randn(moment_dim,CV_sims);

%Draw a uniform set of points to approximate weighted average power
%calculation
WAP_thetas= [rand(1,WAP_sample_size)*(beta_bounds(2)-beta_bounds(1))+beta_bounds(1);
    rand(1,WAP_sample_size)*(alpha_bounds(2)-alpha_bounds(1))+alpha_bounds(1)];

%Calculate numerator in WAP statistic
lik_num_store=zeros(WAP_sample_size,1);
for n1=1:WAP_sample_size
    theta1=WAP_thetas(:,n1);
    gmat_1=gmat(tau,theta1(1),theta1(2));
    g_1=mean(gmat_1)';
    gstore(:,n1)=g_1;
    Sigma_11=cov(gmat_1);
    Sigma_11_inv=pinv(Sigma_11);
    
    lik_num_store(n1,1)=exp(-0.5*obs*g_1'*Sigma_11_inv*g_1);
end
lik_num=mean(lik_num_store);

beta_grid=beta_bounds(1):grid_step:beta_bounds(2);
alpha_grid=alpha_bounds(1):grid_step:alpha_bounds(2);

lik_store=zeros(length(alpha_grid),length(beta_grid));
p_val_store=zeros(length(alpha_grid),length(beta_grid));
init_test_store=zeros(length(alpha_grid),length(beta_grid));
S_store2=zeros(length(alpha_grid),length(beta_grid));

%Loop over parameter space to invert test
for n_alpha=1:length(alpha_grid)
    n_alpha/length(alpha_grid)
    for n_beta=1:length(beta_grid)
        theta0=[beta_grid(n_beta); alpha_grid(n_alpha)];
        
    gmat_0=gmat(tau,theta0(1),theta0(2));
    g_0=mean(gmat_0)';
    Sigma_00=cov(gmat_0);
    Sigma_00_inv=pinv(Sigma_00);
    Sigma_00_rt=Sigma_00^.5;
    lik_store(n_alpha,n_beta)=exp(-0.5*obs*g_0'*Sigma_00_inv*g_0);
    S_store2(n_alpha,n_beta)=obs*g_0'*Sigma_00_inv*g_0;
    
    if (log(lik_num)-log(lik_store(n_alpha,n_beta)))>0.5*chi2inv(0.95,moment_dim)
        p_val_store(n_alpha,n_beta)=0;
        init_test_store(n_alpha,n_beta)=1;
    else
        %Compute and store likelihood over simulation draws
        sim_lik_rat_store=zeros(WAP_sample_size,CV_sims);
        lik_rat_store=zeros(WAP_sample_size,1);
        gstore=zeros(moment_dim,WAP_sample_size);
        parfor n1=1:WAP_sample_size
            theta1=WAP_thetas(:,n1);
            gmat_1=gmat(tau,theta1(1),theta1(2));
            g_1=mean(gmat_1)';
            gstore(:,n1)=g_1;
            Sigma_10=size(gmat_1,1)^-1*gmat_1'*gmat_0-g_1*g_0';
            Sigma_11=cov(gmat_1);
            Sigma_11_inv=pinv(Sigma_11);
            Sigma_11_inv_rt=Sigma_11_inv^0.5;
            
            lik_rat_store(n1,1)=exp(-0.5*obs*g_1'*Sigma_11_inv*g_1+0.5*obs*g_0'*Sigma_00_inv*g_0);
            
            psi=Sigma_10*Sigma_00_inv;
            h1=g_1-psi*g_0;
            
            psi_normalized=Sigma_11_inv_rt*psi;
            h1_normalized=Sigma_11_inv_rt*h1;
            
            g_draws=repmat(h1_normalized,1,CV_sims)+psi_normalized*Sigma_00_rt*xi_draws/obs^.5;
            sim_lik_rat_store(n1,:)=exp(-0.5*obs*sum(g_draws.^2,1)+0.5*sum(xi_draws.^2,1));
        end
        lik_rat=mean(lik_rat_store);
        sim_lik_rat=mean(sim_lik_rat_store,1);
        p_val_store(n_alpha,n_beta)=mean(lik_rat<sim_lik_rat);
    end   
    end
end

%% Generate plots
close all
rng(2)

plot_obs=500;

figure(1)
subplot(2,2,1)
sample_ID=randsample(size(theta_store,2),plot_obs,false);
theta_sample=theta_store(:,sample_ID);
scatter(theta_sample(1,:),theta_sample(2,:));
title('(a) Sample from Quasi-Posterior')
ylim([0,30])
xlim([-10,30])
ylabel('\alpha','FontSize',16)
xlabel('\beta','FontSize',16)

subplot(2,2,2)
GMM_loc=lik_store==max(max(lik_store));
GMM_betas=beta_grid(sum(GMM_loc,1)>0);
GMM_alphas=alpha_grid(sum(GMM_loc,2)>0);
GMM=[mean(GMM_betas);mean(GMM_alphas)];
post_mean=mean(theta_store,2);
histogram(theta_store(1,:),'Normalization','pdf','Edgecolor','none')
hold on
xline(post_mean(1),'--r','Linewidth',1)
xline(GMM(1),'--k','Linewidth',2)
hold off
xlim([-10,30])
xlabel('\beta','FontSize',16)
ylabel('Density','FontSize',16)
title('(b) Quasi-Posterior for \beta')
legend('Quasi-Posterior','Posterior Mean','GMM')

subplot(2,2,3)
HPD_crit=quantile(log_obj_store,1-0.95);
HPD_CR=(lik_store>=exp(HPD_crit));
contourf(beta_grid,alpha_grid,HPD_CR,[1 1],'-b','Linewidth',0.5)
hold on
plot(post_mean(1),post_mean(2),'*r','Markersize',10,'Linewidth',2)
legend('95% HPD Set','Quasi-Posterior Mean','Location','NW')
hold off
xlabel('\beta','FontSize',16)
ylabel('\alpha','FontSize',16)
title('(c) Highest Posterior Density Set')

subplot(2,2,4)
WAP_CS=p_val_store>=0.05;
contourf(beta_grid,alpha_grid,WAP_CS,[1,1],'-b','Linewidth',0.5)
hold on
plot(GMM(1),GMM(2),'*k','Markersize',10,'Linewidth',2)
legend('95% Conditional Confidence Set','GMM Estimate','Location','NW')
hold off
hold off
xlabel('\beta','FontSize',16)
ylabel('\alpha','FontSize',16)
title('(d) Conditional Confidence Set')


figure(2)
subplot(1,2,1)
HPD_crit=quantile(log_obj_store,1-0.95);
HPD_CR=(lik_store>=exp(HPD_crit));
contourf(beta_grid,alpha_grid,HPD_CR,[1 1],'-b','Linewidth',0.5)
hold on
plot(post_mean(1),post_mean(2),'*r','Markersize',10,'Linewidth',2)
legend('95% HPD Set','Quasi-Posterior Mean','Location','NW')
hold off
xlabel('\beta','FontSize',16)
ylabel('\alpha','FontSize',16)
title('(c) Highest Posterior Density Set')

subplot(1,2,2)
WAP_CS=p_val_store>=0.05;
contourf(beta_grid,alpha_grid,WAP_CS,[1,1],'-b','Linewidth',0.5)
hold on
plot(GMM(1),GMM(2),'*k','Markersize',10,'Linewidth',2)
legend('95% Conditional Confidence Set','GMM Estimate','Location','NW')
hold off
hold off
xlabel('\beta','FontSize',16)
ylabel('\alpha','FontSize',16)
title('(d) Conditional Confidence Set')